package kz.arta.demo.utils;

import kz.arta.demo.objects.Contact;
import kz.arta.demo.objects.RegistryData;
import kz.arta.demo.objects.RegistrySearchResultDTO;
import kz.arta.synergy.forms.common.object.ASFData;
import kz.arta.synergy.forms.common.object.ASFDataWrapperExt;
import kz.arta.synergy.forms.common.util.JsonUtil;
import kz.arta.synergy.forms.common.util.rest.operations.AsfDataApi;
import kz.arta.util.index.elastic.HttpBasicOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

/**
 * User: Mikhail Milyutin
 * Date: 25.05.17.
 * Time: 14:41.
 */
public class UserSearchService {
    private static final Logger LOGGER = LoggerFactory.getLogger(UserSearchService.class);

    public static String getUserByNumber(String number) {
        HttpURLConnection connection = null;
        try {

            //TODO: находим userid пользователя, которому нам нужно скинуть уведомления
            URL url = new URL("http://test-kzpitsm.arta.pro/Synergy/rest/api/filecabinet/get_by_field_value?formCode=" + PropCodes.itsm_card_userCard + "&fieldName=" + PropCodes.user_card_initial_number_field + "&value=" + number);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.setRequestProperty("Accept", "application/json; charset=utf-8");

            conn.setRequestProperty("Authorization", PropsUtil.getAuthEncoded());

            String output;
            StringBuilder result = new StringBuilder();

            BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));

            while ((output = br.readLine()) != null) {
                result.append(output);
            }

            conn.disconnect();

            LOGGER.info("[API.RESP] " + result.toString());

            if (!result.toString().isEmpty()) {
                return result.toString();
            }
        } catch (IOException e) {
            LOGGER.error(e.getMessage(), e);
        } finally {
            if (connection != null) {
                connection.disconnect();
            }
        }
        return null;
    }

    public static Contact getContactAndOrganization(String number) {
        RegistryData data = findRegistryRecord(number, PropCodes.crm_form_contact_phone_phone, PropCodes.crm_registry_contacts);
        Contact contact = new Contact(number);
        if (data != null) {
            getContactData(data, contact);
        }
        data = findRegistryRecord(number, PropCodes.crm_form_account_phone_phone, PropCodes.crm_registry_accounts);
        if (data != null) {
            getOrgData(data, contact);
        }
        if(contact.getDocumentID() == null && contact.getOrgDocumentID() == null){
            return null;
        }
        return contact;
    }

    private static void getOrgData(RegistryData registryData, Contact contact) {
        AsfDataApi asfDataApi = new AsfDataApi(PropsUtil.getAddress(), PropsUtil.getAuthEncoded());
        try {
            ASFDataWrapperExt ext = asfDataApi.getAsfData(registryData.getDataUUID());
            ASFData.Data data = ext.getData(PropCodes.crm_form_account_main_name);
            contact.setOrgName(data.getValue());
            contact.setOrgDocumentID(registryData.getDocumentID());
        } catch (IOException exc) {
            LOGGER.error(exc.getMessage(), exc);
        }
    }

    private static void getContactData(RegistryData registryData, Contact contact) {
        AsfDataApi asfDataApi = new AsfDataApi(PropsUtil.getAddress(), PropsUtil.getAuthEncoded());
        try {
            ASFDataWrapperExt ext = asfDataApi.getAsfData(registryData.getDataUUID());

            ASFData.Data data = ext.getData(PropCodes.crm_form_contact_main_organization);
            if(data != null){
                contact.setOrgName(data.getValue());
                contact.setOrgDocumentID(data.getKey());
            }

            data = ext.getData(PropCodes.crm_form_contact_main_fullName);
            contact.setName(data.getValue());
            contact.setDocumentID(registryData.getDocumentID());
        } catch (IOException exc) {
            LOGGER.error(exc.getMessage(), exc);
        }
    }

    private static RegistryData findRegistryRecord(String number, String field, String registryCode) {
        HttpURLConnection connection = null;
        try {
            HttpBasicOperation bo = new HttpBasicOperation();
            connection = bo.openPostConnection(new URL(PropsUtil.getAddress() + "/rest/api/registry/data_ext_post"), PropsUtil.getAuthEncoded());
            if (connection == null) {
                throw new IllegalStateException("connection is null");
            }
            connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            DataOutputStream request = new DataOutputStream(connection.getOutputStream());
            request.writeBytes("registryCode=" + registryCode);
            request.writeBytes("&");
            request.writeBytes("loadData=false");
            request.writeBytes("&");
            request.writeBytes("field=" + field + "&condition=TEXT_EQUALS&value=" + number);
            request.flush();
            request.close();

            RegistrySearchResultDTO records = JsonUtil.getMapper().readValue(connection.getInputStream(), RegistrySearchResultDTO.class);
            if (!records.getData().isEmpty()) {
                return records.getData().get(records.getData().size() - 1);
            }
        } catch (IOException e) {
            LOGGER.error(e.getMessage(), e);
        } finally {
            if (connection != null) {
                connection.disconnect();
            }
        }
        return null;
    }
}
